"""
The module is used to interact with the Vendi Models API.
You can use this module to list, create, update, import models from huggingface and more.

Models can be used to create finetune that will be exposed as new endpoints to consume predictions from.
"""
from .models import Models